package org.ow2.util.deployment.annotations.analyzer.configurator.metadata;

import org.ow2.util.deployment.annotations.analyzer.configurator.BasicConfigurator;
import org.ow2.util.deployment.annotations.analyzer.configurator.ClassConfigurator;
import org.ow2.util.deployment.annotations.analyzer.configurator.FieldConfigurator;
import org.ow2.util.deployment.annotations.analyzer.configurator.MethodConfigurator;
import org.ow2.util.deployment.metadata.ClassMetadata;
import org.ow2.util.deployment.metadata.FieldMetadata;
import org.ow2.util.deployment.metadata.MethodMetadata;
import org.ow2.util.deployment.metadata.structures.JField;
import org.ow2.util.deployment.metadata.structures.JMethod;

/**
 * ClassMetadataConfigurator is a configurator of class specific to ClassMetata.
 * @author Gael Lalire
 * @param <C> class of ClassMetadata
 * @param <M> class of MethodMetadata
 * @param <F> class of FieldMetadata
 */
public abstract class ClassMetadataConfigurator<
    C extends ClassMetadata<C, M, F>,
    M extends MethodMetadata<C, M, F>,
    F extends FieldMetadata<C, M, F>>
        extends BasicConfigurator implements ClassConfigurator {

    /**
     * The class metadata.
     */
    private C classMetadata;

    /**
     * @return the class metadata.
     */
    protected C getClassMetadata() {
        return classMetadata;
    }

    /**
     * Constructor.
     * @param classMetadata a class metadata
     */
    public ClassMetadataConfigurator(final C classMetadata) {
        this.classMetadata = classMetadata;
    }

    /**
     * Provide a method used by ClassConfigurator.
     * @param jMethod signature of method
     * @return the configurator
     */
    public final MethodConfigurator createMethodConfigurator(final JMethod jMethod) {
        return createMethodMetadataConfigurator(jMethod);
    }

    /**
     * You have to override this method.
     * @param jMethod signature of method
     * @return the configurator
     */
    protected abstract MethodConfigurator createMethodMetadataConfigurator(final JMethod jMethod);

    /**
     * Provide a method used by ClassConfigurator.
     * @param jField field signature
     * @return the configurator
     */
    public final FieldConfigurator createFieldConfigurator(final JField jField) {
        return createFieldMetadataConfigurator(jField);
    }

    /**
     * You have to override this method.
     * @param jField signature of field
     * @return the configurator
     */
    protected abstract FieldConfigurator createFieldMetadataConfigurator(final JField jField);

}
