package org.ow2.util.deployment.annotations.analyzer.configurator.metadata;

import org.ow2.util.deployment.annotations.analyzer.configurator.BasicConfigurator;
import org.ow2.util.deployment.annotations.analyzer.configurator.FieldConfigurator;
import org.ow2.util.deployment.metadata.ClassMetadata;
import org.ow2.util.deployment.metadata.FieldMetadata;
import org.ow2.util.deployment.metadata.MethodMetadata;

/**
 * FieldMetadataConfigurator is a configurator of field specific to FieldMetata.
 * @author Gael Lalire
 * @param <C> class of ClassMetadata
 * @param <M> class of MethodMetadata
 * @param <F> class of FieldMetadata
 */
public abstract class FieldMetadataConfigurator<
    C extends ClassMetadata<C, M, F>,
    M extends MethodMetadata<C, M, F>,
    F extends FieldMetadata<C, M, F>>
        extends BasicConfigurator implements FieldConfigurator {

    /**
     * the field metadata.
     */
    private F fieldMetadata;

    /**
     * @return the field metadata
     */
    protected F getFieldMetadata() {
        return fieldMetadata;
    }

    /**
     * Construtor.
     * @param fieldMetadata the field metadata
     */
    public FieldMetadataConfigurator(final F fieldMetadata) {
        this.fieldMetadata = fieldMetadata;
    }

    /**
     * Add the field metadata to his class metadata when configueration ended.
     */
    @Override
    public void configurationComplete() {
        fieldMetadata.getClassMetadata().addFieldMetadata(fieldMetadata);
    }

}
