package org.ow2.util.deployment.annotations.analyzer.configurator.metadata;

import org.ow2.util.deployment.annotations.analyzer.configurator.BasicConfigurator;
import org.ow2.util.deployment.annotations.analyzer.configurator.MethodConfigurator;
import org.ow2.util.deployment.metadata.ClassMetadata;
import org.ow2.util.deployment.metadata.FieldMetadata;
import org.ow2.util.deployment.metadata.MethodMetadata;

/**
 * MethodMetadataConfigurator is a configurator of field specific to MethodMetata.
 * @author Gael Lalire
 * @param <C> class of ClassMetadata
 * @param <M> class of MethodMetadata
 * @param <F> class of FieldMetadata
 */
public abstract class MethodMetadataConfigurator<
    C extends ClassMetadata<C, M, F>,
    M extends MethodMetadata<C, M, F>,
    F extends FieldMetadata<C, M, F>>
        extends BasicConfigurator implements MethodConfigurator {

    /**
     * the method metadata.
     */
    private M methodMetadata;

    /**
     * @return the method metadata
     */
    protected final M getMethodMetadata() {
        return methodMetadata;
    }

    /**
     * Constructor.
     * @param methodMetadata the method metadata
     */
    public MethodMetadataConfigurator(final M methodMetadata) {
        this.methodMetadata = methodMetadata;
    }

    /**
     * Add method metadata to his class metadata when configuration ended.
     */
    @Override
    public void configurationComplete() {
        methodMetadata.getClassMetadata().addMethodMetadata(methodMetadata);
    }

}
