/**
 * EasyBeans
 * Copyright (C) 2006-2007 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MethodMetadata.java 2259 2008-01-28 12:20:29Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.deployment.metadata;

import org.ow2.util.deployment.metadata.structures.JMethod;

/**
 * This class represents the annotation metadata of a method.
 * @author Florent Benoit
 * @param <C> type of class metadata
 * @param <M> type of method metadata
 * @param <F> type of field metadata
 */
public abstract class MethodMetadata<
    C extends ClassMetadata<C, M, F>,
    M extends MethodMetadata<C, M, F>,
    F extends FieldMetadata<C, M, F>>
        extends CommonMetadata {

    /**
     * Logger.
     */
    //private static Log logger = LogFactory.getLog(MethodAnnotationMetadata.class);

    /**
     * Method on which we got metadata.
     */
    private JMethod jMethod = null;

    /**
     * This method is a method from a super class ?<br>
     */
    private boolean inherited = false;

    /**
     * Parent metadata.
     */
    private C classMetadata = null;

    /**
     * Original parent metadata (if method is inherited).
     */
    private C originalClassMetadata = null;

    /**
     * Constructor.
     * @param jMethod the method on which we will set/add metadata
     * @param classMetadata the parent metadata.
     */
    public MethodMetadata(final JMethod jMethod, final C classMetadata) {
        this.jMethod = jMethod;
        this.classMetadata = classMetadata;
    }

    /**
     * @return JMethod object
     */
    public final JMethod getJMethod() {
        return this.jMethod;
    }

    /**
     * Sets the JMethod object (for changing some data).
     * @param jMethod the given JMethod object
     */
    public final void setJMethod(final JMethod jMethod) {
        this.jMethod = jMethod;
    }

    /**
     * @return true if this method is inherited from a super class
     */
    public final boolean isInherited() {
        return inherited;
    }

    /**
     * Sets the inheritance of this method.
     * @param inherited true if method is from a super class
     * @param originalClassMetadata the metadata of the original class (not inherited)
     */
    public final void setInherited(final boolean inherited, final C originalClassMetadata) {
        this.inherited = inherited;
        this.originalClassMetadata  = originalClassMetadata;
    }

    /**
     * @return original parent metadata (class) if inherited.
     */
    public final C getOriginalClassMetadata() {
        return originalClassMetadata;
    }

    /**
     * @return parent metadata (class)
     */
    public final C getClassMetadata() {
        return classMetadata;
    }

    /**
     * @param classMetadata parent metadata (class)
     */
    public final void setClassMetadata(final C classMetadata) {
        this.classMetadata = classMetadata;
    }


}
